@@ -20,23 +20,22 @@
 #ifndef __GENERICFILTER_H__
 #define __GENERICFILTER_H__
 
-#include <qcstring.h>
-#include <qstringlist.h>
+#include <q3cstring.h>
+#include <QStringList>
 
-#include <kprocess.h>
 #include <KoFilter.h>
 #include <KoStore.h>
 
 class GenericFilter : public KoFilter
 {
-	Q_OBJECT
+    Q_OBJECT
 
 public:
-    GenericFilter(KoFilter *parent, const char *name, const QStringList&);
+    GenericFilter(QObject *parent, const QStringList&);
     virtual ~GenericFilter() {}
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from,
-                                                const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from,
+            const QByteArray& to);
 private:
     KoFilter::ConversionStatus doImport();
     KoFilter::ConversionStatus doExport();
