@@ -1,5 +1,6 @@
 /* Swinder - Portable library for spreadsheet
    Copyright (C) 2003 Ariya Hidayat <ariya@kde.org>
+   Copyright (C) 2009,2010 Sebastian Sauer <sebsauer@kdab.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -57,12 +58,16 @@ public:
 
     void setStr(const UString& str);
 
+    std::map<unsigned, unsigned> formatRuns() const;
+
+    void setFormatRuns(const std::map<unsigned, unsigned>& formatRuns);
+
     // space allocate for the string, not length (use string.length() for that)
     unsigned size() const;
     void setSize(unsigned size);   // HACKS
 
 
-    static EString fromUnicodeString(const void* p, bool longString, unsigned maxsize = 0, unsigned continuePosition = 0xFFFFFFFF);
+    static EString fromUnicodeString(const void* p, bool longString, unsigned maxsize = 0, const unsigned *continuePositions = 0, unsigned continuePositionsOffset = 0);
 
     static EString fromSheetName(const void* p, unsigned maxsize = 0);
 
@@ -244,7 +249,7 @@ public:
     /**
      * Creates a new BOF record.
      */
-    BOFRecord();
+    BOFRecord(Swinder::Workbook *book);
 
     /**
       Destroys the record.
@@ -304,7 +309,7 @@ public:
         return this->id;
     }
 
-    ExternBookRecord();
+    ExternBookRecord(Workbook *book);
 
     ~ExternBookRecord();
 
@@ -341,7 +346,7 @@ public:
         return this->id;
     }
 
-    ExternNameRecord();
+    ExternNameRecord(Workbook *book);
 
     ~ExternNameRecord();
 
@@ -387,7 +392,7 @@ public:
     /**
      * Creates a new FILEPASS record.
      */
-    FilepassRecord();
+    FilepassRecord(Workbook *book);
 
     /**
      * Destroy the record.
@@ -427,7 +432,7 @@ public:
     /**
      * Creates a new formula record.
      */
-    FormulaRecord();
+    FormulaRecord(Workbook *book);
 
     /**
      * Destroy the record.
@@ -450,7 +455,7 @@ public:
      * Returns true if this formula is a share formula.
      */
     bool isShared() const;
-    
+
     virtual void setData(unsigned size, const unsigned char* data, const unsigned int* continuePositions);
 
     virtual const char* name() const {
@@ -486,7 +491,7 @@ public:
     /**
      * Creates a new shared formula record.
      */
-    SharedFormulaRecord();
+    SharedFormulaRecord(Workbook *book);
 
     /**
      * Destroy the record.
@@ -532,7 +537,7 @@ public:
     /**
      * Creates a new MulRK record.
      */
-    MulRKRecord();
+    MulRKRecord(Workbook *book);
 
     /**
      * Destroys the record.
@@ -599,6 +604,7 @@ private:
 class NameRecord : public Record
 {
 public:
+    FormulaToken m_formula;
 
     static const unsigned int id;
 
@@ -606,7 +612,7 @@ public:
         return this->id;
     }
 
-    NameRecord();
+    NameRecord(Workbook *book);
 
     ~NameRecord();
 
@@ -649,7 +655,7 @@ public:
     /**
      * Creates a new RK record.
      */
-    RKRecord();
+    RKRecord(Workbook *book);
 
     /**
      * Destroys the record.
@@ -742,7 +748,7 @@ public:
     /**
      * Creates a new Label record.
      */
-    RStringRecord();
+    RStringRecord(Workbook *book);
 
     /**
      * Destroys the record.
@@ -801,7 +807,7 @@ public:
     /**
      * Creates a new SST record.
      */
-    SSTRecord();
+    SSTRecord(Workbook *book);
 
     /**
      * Destroys the record.
@@ -822,6 +828,12 @@ public:
      */
     UString stringAt(unsigned index) const;
 
+    /**
+      Returnsformat runs for the string at specified index.
+      Format runs are a mapping from character-index -> font index.
+     */
+    std::map<unsigned,unsigned> formatRunsAt(unsigned index) const;
+
     virtual const char* name() const {
         return "SST";
     }
@@ -842,10 +854,7 @@ class ObjRecord : public Record
 public:
     Object *m_object;
     static const unsigned id;
-    static Record *createRecord() {
-        return new ObjRecord;
-    }
-    ObjRecord();
+    ObjRecord(Workbook *book);
     virtual ~ObjRecord();
     virtual unsigned rtti() const {
         return this->id;
@@ -857,6 +866,65 @@ public:
     virtual void setData(unsigned size, const unsigned char* data, const unsigned* continuePositions);
 };
 
+class TxORecord : public Record
+{
+public:
+    UString m_text;
+    static const unsigned id;
+    TxORecord(Workbook *book);
+    virtual ~TxORecord();
+    virtual unsigned rtti() const {
+        return this->id;
+    }
+    virtual const char* name() const {
+        return "TxO";
+    }
+    virtual void dump(std::ostream&) const;
+    virtual void setData(unsigned size, const unsigned char* data, const unsigned* continuePositions);
+};
+
+class MsoDrawingRecord : public Record, public DrawingObject
+{
+public:
+    static const unsigned id;
+    MsoDrawingRecord(Workbook *book) : Record(book) {}
+    virtual ~MsoDrawingRecord() {}
+    virtual unsigned rtti() const {
+        return this->id;
+    }
+    virtual const char* name() const {
+        return "MsoDrawing";
+    }
+    virtual void dump(std::ostream&) const;
+    virtual void setData(unsigned size, const unsigned char* data, const unsigned* continuePositions);
+};
+
+class MsoDrawingBlibItem
+{
+public:
+    std::string id;
+    std::string filename;
+    explicit MsoDrawingBlibItem() {}
+};
+
+class MsoDrawingGroupRecord : public Record, private DrawingObject
+{
+public:
+    std::vector< MsoDrawingBlibItem* > m_items;
+
+    static const unsigned id;
+    MsoDrawingGroupRecord(Workbook *book);
+    virtual ~MsoDrawingGroupRecord();
+    virtual unsigned rtti() const {
+        return this->id;
+    }
+    virtual const char* name() const {
+        return "MsoDrawingGroup";
+    }
+    virtual void dump(std::ostream&) const;
+    virtual void setData(unsigned size, const unsigned char* data, const unsigned* continuePositions);
+};
+
 /**
   Class XFRecord holds information of XF (eXtended Format) which specifies
   many kind of properties of a specific cell. It will be referred
@@ -876,7 +944,7 @@ public:
     /**
      * Creates a new XF record.
      */
-    XFRecord();
+    XFRecord(Workbook *book);
 
     /**
      * Creates a copy of XF record.
@@ -1001,11 +1069,11 @@ public:
     const char* horizontalAlignmentAsString() const;
 
     enum {
-        Top = 0,
-        VCentered = 1,
-        Bottom = 2,
-        VJustified = 3,
-        VDistributed = 4
+        Top = 1,
+        VCentered = 2,
+        Bottom = 3,
+        VJustified = 4,
+        VDistributed = 5
     };
 
     /**
@@ -1357,6 +1425,25 @@ private:
     Private* d;
 };
 
+class Picture
+{
+public:
+    std::string m_id;
+    std::string m_filename;
+    unsigned long m_colL, m_dxL, m_rwT, m_dyT, m_colR, m_dxR, m_rwB, m_dyB;
+    Picture(MsoDrawingRecord *record, MsoDrawingBlibItem *item) {
+        m_id = item->id;
+        m_filename = item->filename;
+        m_colL = record->m_colL;
+        m_dxL = record->m_dxL;
+        m_rwT = record->m_rwT;
+        m_dyT = record->m_dyT;
+        m_colR = record->m_colR;
+        m_dxR = record->m_dxR;
+        m_rwB = record->m_rwB;
+        m_dyB = record->m_dyB;
+    }
+};
 
 class ExcelReader
 {
