@@ -3,6 +3,7 @@
    Copyright (C) 2002 David Faure <faure@kde.org>
    Copyright (C) 2008 Benjamin Cail <cricketc@gmail.com>
    Copyright (C) 2009 Inge Wallin   <inge@lysator.liu.se>
+   Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the Library GNU General Public
@@ -27,6 +28,7 @@
 #include "texthandler.h"
 #include "graphicshandler.h"
 #include "versionmagic.h"
+#include "drawstyle.h"
 
 #include <KoUnit.h>
 #include <KoPageLayout.h>
@@ -43,9 +45,10 @@
 #include <KoStore.h>
 #include <KoFilterChain.h>
 #include <KoFontFace.h>
-//Added by qt3to4:
+
 #include <QList>
 #include <QBuffer>
+#include <QColor>
 
 Document::Document(const std::string& fileName, KoFilterChain* chain, KoXmlWriter* bodyWriter,
                    KoGenStyles* mainStyles, KoXmlWriter* metaWriter, KoStore* store, KoXmlWriter* manifestWriter)
@@ -55,6 +58,7 @@ Document::Document(const std::string& fileName, KoFilterChain* chain, KoXmlWrite
         , m_pictureHandler(new KWordPictureHandler(this, bodyWriter,
                            manifestWriter, store,
                            mainStyles))
+        , m_drawingHandler(new KWordDrawingHandler(this,mainStyles,bodyWriter))
         , m_chain(chain)
         , m_parser(wvWare::ParserFactory::createParser(fileName))
         , m_headerFooters(0)
@@ -68,12 +72,14 @@ Document::Document(const std::string& fileName, KoFilterChain* chain, KoXmlWrite
         , m_metaWriter(0)
         , m_masterStyle(0)
         , m_pageLayoutStyle(0)
-        , m_writer(0)
+        , m_headerWriter(0)
         , m_hasHeader(false)
         , m_hasFooter(false)
         , m_buffer(0)
         , m_bufferEven(0)
         , m_headerCount(0)
+        , m_writeMasterStyleName(false)
+        , m_writingHeader(false)
 {
     kDebug(30513);
     if (m_parser) { // 0 in case of major error (e.g. unsupported format)
@@ -82,7 +88,7 @@ Document::Document(const std::string& fileName, KoFilterChain* chain, KoXmlWrite
         m_metaWriter = metaWriter; //pointer for writing to meta.xml
         m_buffer = 0; //set pointers to 0
         m_bufferEven = 0;
-        m_writer = 0;
+        m_headerWriter = 0;
 
         m_textHandler  = new KWordTextHandler(m_parser, bodyWriter, mainStyles);
         m_textHandler->setDocument(this);
@@ -92,12 +98,18 @@ Document::Document(const std::string& fileName, KoFilterChain* chain, KoXmlWrite
                 this, SLOT(slotSubDocFound(const wvWare::FunctorBase*, int)));
         connect(m_textHandler, SIGNAL(footnoteFound(const wvWare::FunctorBase*, int)),
                 this, SLOT(slotFootnoteFound(const wvWare::FunctorBase*, int)));
+        connect(m_textHandler, SIGNAL(annotationFound(const wvWare::FunctorBase*,int)),
+                this, SLOT(slotAnnotationFound(const wvWare::FunctorBase*, int)));
         connect(m_textHandler, SIGNAL(headersFound(const wvWare::FunctorBase*, int)),
                 this, SLOT(slotHeadersFound(const wvWare::FunctorBase*, int)));
         connect(m_textHandler, SIGNAL(tableFound(KWord::Table*)),
                 this, SLOT(slotTableFound(KWord::Table*)));
         connect(m_textHandler, SIGNAL(pictureFound(const QString&, const QString&, KoXmlWriter*, const wvWare::FunctorBase*)),
                 this, SLOT(slotPictureFound(const QString&, const QString&, KoXmlWriter*, const wvWare::FunctorBase*)));
+        connect(m_textHandler, SIGNAL(drawingFound(unsigned int , KoXmlWriter* )),
+                this, SLOT(slotDrawingFound(unsigned int , KoXmlWriter* )));
+        connect(m_drawingHandler, SIGNAL(textBoxFound(uint , KoXmlWriter* )),
+                this, SLOT(slotTextBoxFound(uint , KoXmlWriter* )));
 
         m_parser->setSubDocumentHandler(this);
         m_parser->setTextHandler(m_textHandler);
@@ -105,6 +117,9 @@ Document::Document(const std::string& fileName, KoFilterChain* chain, KoXmlWrite
 #ifdef IMAGE_IMPORT
         m_parser->setPictureHandler(m_pictureHandler);
 #endif
+        m_drawingHandler->init(m_parser->getDrawings(),m_parser->getTable(),m_parser->fib());
+        m_parser->setDrawingHandler(m_drawingHandler);
+
         m_parser->setInlineReplacementHandler(m_replacementHandler);
 
         processStyles();
@@ -138,23 +153,61 @@ void Document::finishDocument()
         QString contents = QString::fromUtf8(m_buffer->buffer(), m_buffer->buffer().size());
         m_masterStyle->addChildElement(QString::number(m_headerCount), contents);
         m_oddOpen = false;
-        m_textHandler->m_headerWriter = 0;
-        delete m_writer;
-        m_writer = 0;
+        delete m_headerWriter;
+        m_headerWriter = 0;
         delete m_buffer;
         m_buffer = 0;
         //we're done with this header, so reset to false
-        m_textHandler->m_writingHeader = false;
+        m_writingHeader = false;
     }
 
-    /*
-        const wvWare::Word97::DOP& dop = m_parser->dop();
-        //"tabStopValue", (double)dop.dxaTab / 20.0
-        //dop.nFtn = initial footnote number for document, starts at 1
-        //Conversion::numberFormatCode(dop.nfcFtnRef2)
-        //dop.nEdn = initial endnote number for document, starts at 1
-        //Conversion::numberFormatCode(dop.nfcEdnRef2)
+    const wvWare::Word97::DOP& dop = m_parser->dop();
 
+    m_initialFootnoteNumber = dop.nFtn;
+    m_initialEndnoteNumber = dop.nEdn;
+
+    //"tabStopValue", (double)dop.dxaTab / 20.0
+
+    Q_ASSERT(m_mainStyles);
+    if (m_mainStyles) {
+
+        QString footnoteConfig("<text:notes-configuration "
+                               "text:note-class=\"footnote\" "
+                               "text:default-style-name=\"Footnote\" "
+                               "text:citation-style-name=\"Footnote_20_Symbol\" "
+                               "text:citation-body-style-name=\"Footnote_20_anchor\" "
+                               "text:master-page-name=\"Footnote\" "
+                               "style:num-format=\"%1\" "
+                               "text:start-value=\"%2\" "
+                               "text:footnotes-position=\"page\" "
+                               "text:start-numbering-at=\"%3\" "
+                               "/>");
+
+        m_mainStyles->insertRawOdfStyles(KoGenStyles::DocumentStyles,
+                                         footnoteConfig.arg(Conversion::numberFormatCode(dop.nfcFtnRef2))
+                                                       .arg(m_initialFootnoteNumber)
+                                                       .arg(Conversion::rncToStartNumberingAt(dop.rncFtn))
+                                                       .toLatin1());
+
+        // ms-word has start-numbering-at (rncEdn) for endnotes, but ODF doesn't really support it
+        QString endnoteConfig("<text:notes-configuration "
+                              "text:note-class=\"endnote\" "
+                              "text:default-style-name=\"Endnote\" "
+                              "text:citation-style-name=\"Endnote_20_Symbol\" "
+                              "text:citation-body-style-name=\"Endnote_20_anchor\" "
+                              "text:master-page-name=\"Endnote\" "
+                              "style:num-format=\"%1\" "
+                              "text:start-value=\"%2\" "
+                              //"text:start-numbering-at=\"%3\" "
+                              "/>");
+
+        m_mainStyles->insertRawOdfStyles(KoGenStyles::DocumentStyles,
+                                         endnoteConfig.arg(Conversion::numberFormatCode(dop.nfcEdnRef2))
+                                                      .arg(m_initialEndnoteNumber)
+//                                                           .arg(Conversion::rncToStartNumberingAt(dop.rncEdn))
+                                                      .toLatin1());
+    }
+    /*
         QDomElement elementDoc = m_mainDocument.documentElement();
 
         QDomElement element;
@@ -173,16 +226,6 @@ void Document::finishDocument()
         element.setAttribute("tabStopValue", (double)dop.dxaTab / 20.0 );
         elementDoc.appendChild(element);
 
-        element = m_mainDocument.createElement("FOOTNOTESETTING");
-        elementDoc.appendChild(element);
-        element.setAttribute( "start", dop.nFtn ); // initial footnote number for document. Starts at 1.
-        element.setAttribute( "type", Conversion::numberFormatCode( dop.nfcFtnRef2 ) );
-
-        element = m_mainDocument.createElement("ENDNOTESETTING");
-        elementDoc.appendChild(element);
-        element.setAttribute( "start", dop.nEdn ); // initial endnote number for document. Starts at 1.
-        element.setAttribute( "type", Conversion::numberFormatCode( dop.nfcEdnRef2 ) );
-
         // Done at the end: write the type of headers/footers,
         // depending on which kind of headers and footers we received.
         QDomElement paperElement = elementDoc.namedItem("PAPER").toElement();
@@ -257,7 +300,7 @@ void Document::processStyles()
 
             //create this style & add formatting info to it
             kDebug(30513) << "creating ODT paragraphstyle" << name;
-            KoGenStyle userStyle(KoGenStyle::StyleUser, "paragraph");
+            KoGenStyle userStyle(KoGenStyle::ParagraphStyle, "paragraph");
             userStyle.addAttribute("style:display-name", displayName);
 
             const wvWare::Style* parentStyle = styles.styleByIndex(style->m_std->istdBase);
@@ -269,7 +312,7 @@ void Document::processStyles()
             //set font name in style
             QString fontName = m_textHandler->getFont(style->chp().ftcAscii);
             if (!fontName.isEmpty()) {
-                m_mainStyles->addFontFace(KoFontFace(fontName));
+                m_mainStyles->insertFontFace(KoFontFace(fontName));
                 userStyle.addProperty(QString("style:font-name"), fontName, KoGenStyle::TextType);
             }
 
@@ -281,12 +324,12 @@ void Document::processStyles()
 
             // Add style to main collection, using the name that it
             // had in the .doc.
-            QString actualName = m_mainStyles->lookup(userStyle, name, KoGenStyles::DontForceNumbering);
+            QString actualName = m_mainStyles->insert(userStyle, name, KoGenStyles::DontAddNumberToName);
             kDebug(30513) << "added style " << actualName;
         } else if (style && style->type() == wvWare::Style::sgcChp) {
             //create this style & add formatting info to it
             kDebug(30513) << "creating ODT textstyle" << name;
-            KoGenStyle userStyle(KoGenStyle::StyleUser, "text");
+            KoGenStyle userStyle(KoGenStyle::ParagraphStyle, "text");
             userStyle.addAttribute("style:display-name", displayName);
 
             const wvWare::Style* parentStyle = styles.styleByIndex(style->m_std->istdBase);
@@ -299,7 +342,7 @@ void Document::processStyles()
             // the design is such that the styles are modified during processing of text runs
 
             //add style to main collection, using the name that it had in the .doc
-            QString actualName = m_mainStyles->lookup(userStyle, name, KoGenStyles::DontForceNumbering);
+            QString actualName = m_mainStyles->insert(userStyle, name, KoGenStyles::DontAddNumberToName);
             kDebug(30513) << "added style " << actualName;
         }
     }
@@ -347,21 +390,29 @@ void Document::slotSectionFound(wvWare::SharedPtr<const wvWare::Word97::SEP> sep
     kDebug(30513) ;
     //need to add master style to m_mainStyle
     kDebug(30513) << "creating master style for this section";
-    m_masterStyle = new KoGenStyle(KoGenStyle::StyleMaster); //for header/footer stuff
-    QString masterStyleName("section");
-    m_masterStyle->addAttribute("style:display-name", masterStyleName.append(QString::number(m_textHandler->m_sectionNumber)));
-    m_masterStyleName = m_mainStyles->lookup(*m_masterStyle, masterStyleName, KoGenStyles::DontForceNumbering);
-    delete m_masterStyle; //delete the object since we've added it to the collection
+    m_masterStyle = new KoGenStyle(KoGenStyle::MasterPageStyle); //for header/footer stuff
 
-    //set master style name in m_textHandler because that's where we'll write it
-    m_textHandler->m_masterStyleName = m_masterStyleName;
-    //get a pointer to the object in the collection
+    // Kword expects that first section has StyleMaster named "Standard"
+    QString masterStyleName("Standard");
+    if (m_textHandler->m_sectionNumber > 1) {
+        masterStyleName.append(QString::number(m_textHandler->m_sectionNumber));
+    }
+
+    m_masterStyle->addAttribute("style:display-name", masterStyleName);
+    m_masterStyleName = m_mainStyles->insert(*m_masterStyle, masterStyleName, KoGenStyles::DontAddNumberToName);
+    //delete the object since we've added it to the collection
+    delete m_masterStyle;
+    m_masterStyle = 0;
+
+    //Get a pointer to the object in the collection. TODO: Check
+    //sep->fTitlePage, requires a separate master-page.
     m_masterStyle = m_mainStyles->styleForModification(m_masterStyleName);
-    m_textHandler->m_writeMasterStyleName = true;
+    Q_ASSERT(m_masterStyle);
+    m_writeMasterStyleName = true;
 
     // ----------------------
     //create page layout style here
-    m_pageLayoutStyle = new KoGenStyle(KoGenStyle::StylePageLayout);
+    m_pageLayoutStyle = new KoGenStyle(KoGenStyle::PageLayoutStyle);
     QString pageLayoutStyleName("Mpm");
 
     //get width & height in points
@@ -380,6 +431,16 @@ void Document::slotSectionFound(wvWare::SharedPtr<const wvWare::Word97::SEP> sep
     m_pageLayoutStyle->addPropertyPt("fo:margin-left", (double)sep->dxaLeft / 20.0);
     m_pageLayoutStyle->addPropertyPt("fo:margin-right", (double)sep->dxaRight / 20.0);
 
+    DrawStyle ds = m_drawingHandler->getDrawingStyle();
+    // TODO - use ds.fillType to enable fill efects for document background
+
+    // When more complete background color implementation will be added to filter,
+    // PptToOdp::toQColor helper function can be used instead of this conversion
+    MSO::OfficeArtCOLORREF clr = ds.fillColor();
+    QColor color(clr.red,clr.green,clr.blue);
+    m_pageLayoutStyle->addProperty("fo:background-color", color.name());
+
+
     //set the minimum height of header/footer to the full margin minus margin above header
     //TODO the margin between header/footer and text is just hard-coded for now
     QString header("<style:header-style>");
@@ -395,11 +456,6 @@ void Document::slotSectionFound(wvWare::SharedPtr<const wvWare::Word97::SEP> sep
     m_pageLayoutStyle->addProperty("1header-style", header, KoGenStyle::StyleChildElement);
     m_pageLayoutStyle->addProperty("2footer-style", footer, KoGenStyle::StyleChildElement);
 
-    m_pageLayoutStyle->setAutoStyleInStylesDotXml(true);
-
-    pageLayoutStyleName = m_mainStyles->lookup(*m_pageLayoutStyle, pageLayoutStyleName, KoGenStyles::DontForceNumbering);
-    m_masterStyle->addAttribute("style:page-layout-name", pageLayoutStyleName);
-
     // Page borders
     // FIXME: check if we can use fo:border instead of fo:border-left, etc.
     if (sep->brcLeft.brcType != 0) {
@@ -448,8 +504,18 @@ void Document::slotSectionFound(wvWare::SharedPtr<const wvWare::Word97::SEP> sep
                                          (sep->dyaBottom - (sep->brcBottom.dptSpace * 20)) / 20);
     }
 
+    m_pageLayoutStyle->setAutoStyleInStylesDotXml(true);
 
-    // TODO use sep->fEndNote to set the 'use endnotes or footnotes' flag
+    pageLayoutStyleName = m_mainStyles->insert(*m_pageLayoutStyle, pageLayoutStyleName, KoGenStyles::DontAddNumberToName);
+    //delete the object since we've added it to the collection
+    delete m_pageLayoutStyle;
+    m_pageLayoutStyle = 0;
+
+    //Get a pointer to the object in the collection.
+    m_pageLayoutStyle = m_mainStyles->styleForModification(pageLayoutStyleName);
+    Q_ASSERT(m_pageLayoutStyle);
+
+    // TODO: use sep->fEndNote to set the 'use endnotes or footnotes' flag
 }
 
 void Document::slotSectionEnd(wvWare::SharedPtr<const wvWare::Word97::SEP> sep)
@@ -468,12 +534,11 @@ void Document::slotSectionEnd(wvWare::SharedPtr<const wvWare::Word97::SEP> sep)
     } else if (sep->brcBottom.brcType == 0) {
         m_pageLayoutStyle->addPropertyPt("fo:margin-bottom", (double)sep->dyaBottom / 20.0);
     }
-    //insert the page-layout style into the collection,
-    //and get the name it's assigned
-    QString pageLayoutName = m_mainStyles->lookup(*m_pageLayoutStyle, QString("pm"));
-    //set the page-layout-name in the master style
-    m_masterStyle->addAttribute("style:page-layout-name", pageLayoutName);
-    delete m_pageLayoutStyle;
+    //Set the page-layout-name in the master style. TODO: Check
+    //sep->fTitlePage, requires a separate master-page.
+    m_masterStyle->addAttribute("style:page-layout-name", QString("Mpm"));
+
+    //don't need the pointer any more
     m_pageLayoutStyle = 0;
     //reset variables
     m_hasHeader = false;
@@ -496,56 +561,54 @@ void Document::headerStart(wvWare::HeaderData::Type type)
     case wvWare::HeaderData::HeaderFirst:
         m_buffer = new QBuffer();
         m_buffer->open(QIODevice::WriteOnly);
-        m_writer = new KoXmlWriter(m_buffer);
+        m_headerWriter = new KoXmlWriter(m_buffer);
         break;
     case wvWare::HeaderData::HeaderOdd:
         //set up buffer & writer for odd header
         m_buffer = new QBuffer();
         m_buffer->open(QIODevice::WriteOnly);
-        m_writer = new KoXmlWriter(m_buffer);
+        m_headerWriter = new KoXmlWriter(m_buffer);
         m_oddOpen = true;
-        m_writer->startElement("style:header");
+        m_headerWriter->startElement("style:header");
         m_hasHeader = true;
         break;
     case wvWare::HeaderData::HeaderEven:
         //write to the buffer for even headers/footers
         m_bufferEven = new QBuffer();
         m_bufferEven->open(QIODevice::WriteOnly);
-        m_writer = new KoXmlWriter(m_bufferEven);
+        m_headerWriter = new KoXmlWriter(m_bufferEven);
         m_evenOpen = true;
-        m_writer->startElement("style:header-left");
+        m_headerWriter->startElement("style:header-left");
         m_hasHeader = true;
         break;
         //TODO fix first footer
     case wvWare::HeaderData::FooterFirst:
         m_buffer = new QBuffer();
         m_buffer->open(QIODevice::WriteOnly);
-        m_writer = new KoXmlWriter(m_buffer);
+        m_headerWriter = new KoXmlWriter(m_buffer);
         break;
     case wvWare::HeaderData::FooterOdd:
         //set up buffer & writer for odd header
         m_buffer = new QBuffer();
         m_buffer->open(QIODevice::WriteOnly);
-        m_writer = new KoXmlWriter(m_buffer);
+        m_headerWriter = new KoXmlWriter(m_buffer);
         m_oddOpen = true;
-        m_writer->startElement("style:footer");
+        m_headerWriter->startElement("style:footer");
         m_hasFooter = true;
         break;
     case wvWare::HeaderData::FooterEven:
         //write to the buffer for even headers/footers
         m_bufferEven = new QBuffer();
         m_bufferEven->open(QIODevice::WriteOnly);
-        m_writer = new KoXmlWriter(m_bufferEven);
+        m_headerWriter = new KoXmlWriter(m_bufferEven);
         m_evenOpen = true;
-        m_writer->startElement("style:footer-left");
+        m_headerWriter->startElement("style:footer-left");
         m_hasFooter = true;
         break;
     }
 
-    //tell texthandler we're writing a header
-    m_textHandler->m_writingHeader = true;
-    //and set up the tmp writer so writeFormattedText() writes to styles.xml
-    m_textHandler->m_headerWriter = m_writer;
+    //we're writing a header
+    m_writingHeader = true;
 }
 
 //creates empty frameset element?
@@ -563,17 +626,17 @@ void Document::headerEnd()
     //handle the even flag first, because they'll both be open if the even one is, and
     //  we would want to handle the odd flag when we actually see the odd header/footer
     if (m_evenOpen) {
-        m_writer->endElement(); //style:header/footer-left
+        m_headerWriter->endElement(); //style:header/footer-left
         m_evenOpen = false;
-        delete m_writer;
-        m_writer = 0;
+        delete m_headerWriter;
+        m_headerWriter = 0;
         return;
     }
     if (m_oddOpen) {
-        m_writer->endElement();//style:header/footer
+        m_headerWriter->endElement();//style:header/footer
         //add the even header/footer stuff here
         if (m_bufferEven) {
-            m_writer->addCompleteElement(m_bufferEven);
+            m_headerWriter->addCompleteElement(m_bufferEven);
             delete m_bufferEven;
             m_bufferEven = 0;
         }
@@ -581,18 +644,17 @@ void Document::headerEnd()
         m_masterStyle->addChildElement(QString::number(m_headerCount), contents);
         m_oddOpen = false;
     }
-    m_textHandler->m_headerWriter = 0;
-    delete m_writer;
-    m_writer = 0;
+    delete m_headerWriter;
+    m_headerWriter = 0;
     delete m_buffer;
     m_buffer = 0;
     //we're done with this header, so reset to false
-    m_textHandler->m_writingHeader = false;
+    m_writingHeader = false;
 }
 
 void Document::footnoteStart()
 {
-    kDebug(30513) ;
+    kDebug(30513);
 }
 
 void Document::footnoteEnd()
@@ -600,6 +662,16 @@ void Document::footnoteEnd()
     kDebug(30513);
 }
 
+
+void Document::annotationStart()
+{
+}
+
+void Document::annotationEnd()
+{
+}
+
+
 //disable this for now - we should be able to do everything in TableHandler
 //create frame for the table cell?
 //void Document::slotTableCellStart( int row, int column, int rowSpan, int columnSpan, const QRectF& cellRect, const QString& tableName, const wvWare::Word97::BRC& brcTop, const wvWare::Word97::BRC& brcBottom, const wvWare::Word97::BRC& brcLeft, const wvWare::Word97::BRC& brcRight, const wvWare::Word97::SHD& shd )
@@ -708,6 +780,14 @@ void Document::slotFootnoteFound(const wvWare::FunctorBase* functor, int data)
     delete subdoc.functorPtr;
 }
 
+void Document::slotAnnotationFound(const wvWare::FunctorBase* functor, int data)
+{
+    kDebug(30513) ;
+    SubDocument subdoc(functor, data, QString(), QString());
+    (*subdoc.functorPtr)();
+    delete subdoc.functorPtr;
+}
+
 void Document::slotHeadersFound(const wvWare::FunctorBase* functor, int data)
 {
     kDebug(30513) ;
@@ -755,6 +835,27 @@ void Document::slotPictureFound(const QString& frameName, const QString& picture
     }
 }
 
+void Document::slotDrawingFound(unsigned int globalCP, KoXmlWriter* writer)
+{
+    kDebug(30513) ;
+    //if we have a temp writer, tell the drawingHandler
+    if (writer) {
+        m_drawingHandler->setBodyWriter(writer);
+    }
+    m_drawingHandler->drawingData(globalCP);
+
+    if (writer) {
+        m_drawingHandler->setBodyWriter(m_bodyWriter);
+    }
+}
+
+void Document::slotTextBoxFound( uint lid, KoXmlWriter* /*writer*/)
+{
+    kDebug(30513) ;
+
+    m_parser->parseTextBox(lid);
+}
+
 //process through all the subDocs and the tables
 void Document::processSubDocQueue()
 {
