@@ -39,7 +39,7 @@ Paragraph::Paragraph(KoGenStyles* mainStyles, bool inStylesDotXml, bool isHeadin
 {
     kDebug(30513);
     m_mainStyles = mainStyles;
-    m_odfParagraphStyle = new KoGenStyle(KoGenStyle::StyleAuto, "paragraph");
+    m_odfParagraphStyle = new KoGenStyle(KoGenStyle::ParagraphAutoStyle, "paragraph");
     if (inStylesDotXml) {
         kDebug(30513) << "this paragraph is in styles.xml";
         m_inStylesDotXml = true;
@@ -102,11 +102,16 @@ void Paragraph::addRunOfText(QString text,  wvWare::SharedPtr<const wvWare::Word
         kWarning() << "Couldn't retrieve style for modification!";
     }
 
+    bool suppresFontSize = false;
+    if (m_textStyles.size() == 0 && m_paragraphProperties->pap().dcs.lines > 1) {
+        suppresFontSize = true;
+    }
+
     // Modify the character style if we detect any diff between the
     // chp of the paragraph and the summed chp.
     const wvWare::Style* parentStyle = styles.styleByIndex(msTextStyle->m_std->istdBase);
     if (parentStyle) {
-        applyCharacterProperties(chp, textStyle, m_paragraphStyle);
+        applyCharacterProperties(chp, textStyle, m_paragraphStyle, suppresFontSize);
         //if we have a new font, process that
         const wvWare::Word97::CHP* refChp = &m_paragraphStyle->chp();
         if (!refChp || refChp->ftcAscii != chp->ftcAscii) {
@@ -115,14 +120,14 @@ void Paragraph::addRunOfText(QString text,  wvWare::SharedPtr<const wvWare::Word
             }
         }
     } else {
-        textStyle = new KoGenStyle(KoGenStyle::StyleTextAuto, "text");
+        textStyle = new KoGenStyle(KoGenStyle::TextAutoStyle, "text");
         if (m_inStylesDotXml) {
             textStyle->setAutoStyleInStylesDotXml(true);
         }
         if (!fontName.isEmpty()) {
             textStyle->addProperty(QString("style:font-name"), fontName, KoGenStyle::TextType);
         }
-        applyCharacterProperties(chp, textStyle, m_paragraphStyle);
+        applyCharacterProperties(chp, textStyle, m_paragraphStyle, suppresFontSize);
     }
 
     //add text style to list
@@ -148,7 +153,7 @@ void Paragraph::writeToFile(KoXmlWriter* writer)
     kDebug(30513) << "adding paragraphStyle";
     //add the attribute for our style in <text:p>
     QString styleName("P");
-    styleName = m_mainStyles->lookup(*m_odfParagraphStyle, styleName);
+    styleName = m_mainStyles->insert(*m_odfParagraphStyle, styleName);
     writer->addAttribute("text:style-name", styleName.toUtf8());
 
     //just close the paragraph if there's no content
@@ -174,7 +179,7 @@ void Paragraph::writeToFile(KoXmlWriter* writer)
             //put style into m_mainStyles & get its name
             //kDebug(30513) << m_textStyles[i]->type();
             styleName = 'T';
-            styleName = m_mainStyles->lookup(*m_textStyles[i], styleName);
+            styleName = m_mainStyles->insert(*m_textStyles[i], styleName);
 
             if (oldStyleName != styleName) {
                 if (startedSpan) {
@@ -230,7 +235,7 @@ void Paragraph::openInnerParagraph()
 
     //copy parent and paragraph styles
     m_odfParagraphStyle2 = m_odfParagraphStyle;
-    m_odfParagraphStyle = new KoGenStyle(KoGenStyle::StyleAuto, "paragraph");
+    m_odfParagraphStyle = new KoGenStyle(KoGenStyle::ParagraphAutoStyle, "paragraph");
     m_paragraphStyle2 = m_paragraphStyle;
     m_paragraphProperties2 = m_paragraphProperties;
     m_paragraphProperties = 0;
@@ -353,18 +358,19 @@ void Paragraph::applyParagraphProperties(const wvWare::ParagraphProperties& prop
             //
             // Get the proportion & turn it into a percentage for the
             // attribute.
-            QString proportionalLineSpacing(QString::number(((double)pap.lspd.dyaLine / 240.0)*100.0));
+            QString proportionalLineSpacing(QString::number(((qreal)pap.lspd.dyaLine
+                                                  / (qreal)240.0)*(qreal)120.0));
             style->addProperty("fo:line-height", proportionalLineSpacing.append("%"), KoGenStyle::ParagraphType);
         } else if (pap.lspd.fMultLinespace == 0) {
             // Magnitude of lspd.dyaLine specifies the amount of space
             // that will be provided for lines in the paragraph in twips.
             //
             // See sprmPDyaLine in generator_wword8.htm
-            double value = qAbs((double)pap.lspd.dyaLine / 20.0); // twip -> pt
+            qreal value = qAbs((qreal)pap.lspd.dyaLine / (qreal)20.0); // twip -> pt
             // lspd.dyaLine > 0 means "at least", < 0 means "exactly"
             if (pap.lspd.dyaLine > 0)
                 style->addPropertyPt("fo:line-height-at-least", value, KoGenStyle::ParagraphType);
-            else if (pap.lspd.dyaLine < 0)
+            else if (pap.lspd.dyaLine < 0 && pap.dcs.fdct==0)
                 style->addPropertyPt("fo:line-height", value, KoGenStyle::ParagraphType);
         } else
             kWarning(30513) << "Unhandled LSPD::fMultLinespace value: "
@@ -419,6 +425,20 @@ void Paragraph::applyParagraphProperties(const wvWare::ParagraphProperties& prop
         style->addProperty("fo:border-right", Conversion::setBorderAttributes(pap.brcRight), KoGenStyle::ParagraphType);
     }
 
+    if (!refPap || refPap->dcs.fdct != pap.dcs.fdct || refPap->dcs.lines != pap.dcs.lines) {
+        QBuffer buf;
+        buf.open(QIODevice::WriteOnly);
+        KoXmlWriter tmpWriter(&buf, 3);
+        tmpWriter.startElement("style:drop-cap");
+        tmpWriter.addAttribute("style:lines", pap.dcs.lines);
+        tmpWriter.addAttributePt("style:distance", (qreal)pap.dxaFromText / (qreal)20.0);
+        tmpWriter.addAttribute("style:length", pap.dcs.fdct > 0 ? 1 : 0);
+        tmpWriter.endElement();//style:drop-cap
+        buf.close();
+        QString contents = QString::fromUtf8(buf.buffer(), buf.buffer().size());
+        style->addChildElement("style:drop-cap", contents);
+    }
+
 //TODO introduce diff for tabs too like in: if(!refPap || refPap->fKeep != pap
 
     // Tabulators
@@ -470,7 +490,7 @@ void Paragraph::applyParagraphProperties(const wvWare::ParagraphProperties& prop
     }
 } //end applyParagraphProperties
 
-void Paragraph::applyCharacterProperties(const wvWare::Word97::CHP* chp, KoGenStyle* style, const wvWare::Style* parentStyle)
+void Paragraph::applyCharacterProperties(const wvWare::Word97::CHP* chp, KoGenStyle* style, const wvWare::Style* parentStyle, bool suppressFontSize)
 {
     //if we have a named style, set its CHP as the refChp
     const wvWare::Word97::CHP* refChp;
@@ -486,7 +506,7 @@ void Paragraph::applyCharacterProperties(const wvWare::Word97::CHP* chp, KoGenSt
     }
 
     //hps = font size in half points
-    if (!refChp || refChp->hps != chp->hps) {
+    if (!suppressFontSize && (!refChp || refChp->hps != chp->hps)) {
         style->addPropertyPt(QString("fo:font-size"), (int)(chp->hps / 2), KoGenStyle::TextType);
     }
 
@@ -601,5 +621,20 @@ void Paragraph::applyCharacterProperties(const wvWare::Word97::CHP* chp, KoGenSt
             style->addProperty("style:font-relief", "engraved", KoGenStyle::TextType);
     }
 
+    //fOutline = text is outline if 1
+    if (!refChp || refChp->fOutline != chp->fOutline) {
+        if (chp->fOutline)
+            style->addProperty("style:text-outline", "true", KoGenStyle::TextType);
+        else
+            style->addProperty("style:text-outline", "false", KoGenStyle::TextType);
+    }
+
+
+    //dxaSpace = letterspacing in twips
+    if (!refChp || refChp->dxaSpace != chp->dxaSpace) {
+        double value =  chp->dxaSpace / 20.0; // twips -> pt
+        style->addPropertyPt("fo:letter-spacing", value, KoGenStyle::TextType);
+    }
+    //pctCharwidth = pct stretch doesn't seem to have an ODF ccounterpart but Qt could support it
 }
 
