@@ -61,22 +61,32 @@ public:
     bool             firstColumnIsLabel;
     Qt::Orientation  dataDirection;
     int              dataDimensions;
-    QMap<int, int>   dataMap;
+    
+    QString categoryDataRegion;
+
+    QVector< CellRegion > dataSetRegions;
     
     QList<DataSet*>  dataSets;
     QList<DataSet*>  removedDataSets;
     
     QVector<QRect>   selection;
-};
 
+    bool automaticDataSetCreation;
+};
 
 ChartProxyModel::Private::Private()
 {
     firstRowIsLabel    = false;
     firstColumnIsLabel = false;
     dataDimensions     = 1;
+    automaticDataSetCreation = true;
 
-    dataDirection      = Qt::Vertical; // Apparently the default is columns.
+    // Determines what orientation the data points in a data series
+    // have when multiple data sets are created from one source
+    // region. For example, vertical means that each column in the source
+    // region is assigned to one data series.
+    // Default to Qt::Vertical, as that's what OOo does also.
+    dataDirection      = Qt::Vertical;
 }
 
 ChartProxyModel::Private::~Private()
@@ -100,6 +110,16 @@ ChartProxyModel::~ChartProxyModel()
 }
 
 
+void ChartProxyModel::setAutomaticDataSetCreation( bool enable )
+{
+    d->automaticDataSetCreation = enable;
+}
+
+bool ChartProxyModel::automaticDataSetCreation() const
+{
+    return d->automaticDataSetCreation;
+}
+
 void ChartProxyModel::beginResetModel()
 {
 #if QT_VERSION  >= 0x040600
@@ -118,8 +138,16 @@ void ChartProxyModel::endResetModel()
 
 void ChartProxyModel::rebuildDataMap()
 {
-    invalidateDataSets();
+    if ( !d->automaticDataSetCreation )
+        return;
 
+    invalidateDataSets();
+    d->dataSets = createDataSetsFromRegion( d->removedDataSets );
+}
+
+QList<DataSet*> ChartProxyModel::createDataSetsFromRegion( QList<DataSet*> dataSetsToRecycle )
+{
+    QList<DataSet*> createdDataSets;
     QVector<QRect> dataRegions;
 
     if ( d->selection.isEmpty() ) {
@@ -172,48 +200,59 @@ void ChartProxyModel::rebuildDataMap()
         
         QMapIterator<int, QVector<QRect> > j( sortedRows );
         
-        CellRegion categoryDataRegion;
-        
-        if ( d->firstRowIsLabel && j.hasNext() ) {
+        CellRegion category;
+        if ( ! categoryDataRegion().isEmpty() ) {
+            category = CellRegion( categoryDataRegion() );
+        } else if ( d->firstRowIsLabel && j.hasNext() ) {
             j.next();
             
-            categoryDataRegion = CellRegion( j.value() );
+            category = CellRegion( j.value() );
             if ( d->firstColumnIsLabel )
-                categoryDataRegion.subtract( categoryDataRegion.pointAtIndex( 0 ) );
+                category.subtract( category.pointAtIndex( 0 ) );
         }
         
         while ( j.hasNext() ) {
             j.next();
             
             DataSet *dataSet;
-            if ( createdDataSetCount >= d->dataSets.size() ) {
-                if ( !d->removedDataSets.isEmpty() )
-                    dataSet = d->removedDataSets.takeLast();
-                else
-                    dataSet = new DataSet( this );
-
-                d->dataSets.append( dataSet );
-            }
+            if ( !dataSetsToRecycle.isEmpty() )
+                dataSet = dataSetsToRecycle.takeLast();
             else
-                dataSet = d->dataSets[createdDataSetCount];
+                dataSet = new DataSet( this );
+            createdDataSets.append( dataSet );
+
             dataSet->blockSignals( true );
             
             dataSet->setNumber( createdDataSetCount );
-            dataSet->setColor( defaultDataSetColor( createdDataSetCount ) );
-            
-            CellRegion yDataRegion( j.value() );
+            //dataSet->setColor( defaultDataSetColor( createdDataSetCount ) );
+
             CellRegion labelDataRegion;
-    
+
+            CellRegion xDataRegion;
+            // In case of > 1 data dimensions, x data appears before y data
+            if ( d->dataDimensions > 1 )
+                xDataRegion = CellRegion( j.value() );
+
             //qDebug() << "Creating data set with region" << j.value();
             if ( d->firstColumnIsLabel ) {
-                QPoint labelDataPoint = yDataRegion.pointAtIndex( 0 );
+                CellRegion tmpRegion = CellRegion( j.value() );
+                QPoint labelDataPoint = tmpRegion.pointAtIndex( 0 );
                 labelDataRegion = CellRegion( labelDataPoint );
-                
-                yDataRegion.subtract( labelDataPoint );
             }
+
+            if ( d->dataDimensions > 1 && j.hasNext() )
+                j.next();
             
+            CellRegion yDataRegion( j.value() );
+
+            if ( d->firstColumnIsLabel ) {
+                xDataRegion.subtract( xDataRegion.pointAtIndex( 0 ) );
+                yDataRegion.subtract( yDataRegion.pointAtIndex( 0 ) );
+            }
+
+            dataSet->setXDataRegion( xDataRegion );
             dataSet->setYDataRegion( yDataRegion );
-            dataSet->setCategoryDataRegion( categoryDataRegion );
+            dataSet->setCategoryDataRegion( category );
             dataSet->setLabelDataRegion( labelDataRegion );
             createdDataSetCount++;
             dataSet->blockSignals( false );
@@ -262,62 +301,66 @@ void ChartProxyModel::rebuildDataMap()
         
         QMapIterator<int, QVector<QRect> > j( sortedColumns );
         
-        CellRegion categoryDataRegion;
-        
-        if ( d->firstColumnIsLabel && j.hasNext() ) {
+        CellRegion category;
+        if ( ! categoryDataRegion().isEmpty() ) {
+            category = CellRegion( categoryDataRegion() );
+        } else if ( d->firstColumnIsLabel && j.hasNext() ) {
             j.next();
             
-            categoryDataRegion = CellRegion( j.value() );
+            category = CellRegion( j.value() );
             if ( d->firstRowIsLabel )
-                categoryDataRegion.subtract( categoryDataRegion.pointAtIndex( 0 ) );
+                category.subtract( category.pointAtIndex( 0 ) );
         }
         
         while ( j.hasNext() ) {
             j.next();
-            
+
             DataSet *dataSet;
-            if ( createdDataSetCount >= d->dataSets.size() ) {
-                if ( !d->removedDataSets.isEmpty() )
-                    dataSet = d->removedDataSets.takeLast();
-                else
-                    dataSet = new DataSet( this );
-                d->dataSets.append( dataSet );
-            }
+            if ( !dataSetsToRecycle.isEmpty() )
+                dataSet = dataSetsToRecycle.takeLast();
             else
-                dataSet = d->dataSets[createdDataSetCount];
+                dataSet = new DataSet( this );
+            createdDataSets.append( dataSet );
+
             dataSet->blockSignals( true );
             
             dataSet->setNumber( createdDataSetCount );
-            dataSet->setColor( defaultDataSetColor( createdDataSetCount ) );
+            //dataSet->setColor( defaultDataSetColor( createdDataSetCount ) );
+
+            CellRegion labelDataRegion;
+            
+            CellRegion xDataRegion;
+            // In case of > 1 data dimensions, x data appears before y data
+            if ( d->dataDimensions > 1 )
+                xDataRegion = CellRegion( j.value() );
+
+            //qDebug() << "Creating data set with region" << j.value();
+            if ( d->firstRowIsLabel ) {
+                CellRegion tmpRegion = CellRegion( j.value() );
+                QPoint labelDataPoint = tmpRegion.pointAtIndex( 0 );
+                labelDataRegion = CellRegion( labelDataPoint );
+            }
+
+            if ( d->dataDimensions > 1 && j.hasNext() )
+                j.next();
             
             CellRegion yDataRegion( j.value() );
-            CellRegion labelDataRegion;
-    
-            //qDebug() << "Creating data set with region " << j.value();
+
             if ( d->firstRowIsLabel ) {
-                QPoint labelDataPoint = yDataRegion.pointAtIndex( 0 );
-                labelDataRegion = CellRegion( labelDataPoint );
-                
-                yDataRegion.subtract( labelDataPoint );
+                xDataRegion.subtract( xDataRegion.pointAtIndex( 0 ) );
+                yDataRegion.subtract( yDataRegion.pointAtIndex( 0 ) );
             }
-            
+
+            dataSet->setXDataRegion( xDataRegion );
             dataSet->setYDataRegion( yDataRegion );
             dataSet->setLabelDataRegion( labelDataRegion );
-            dataSet->setCategoryDataRegion( categoryDataRegion );
+            dataSet->setCategoryDataRegion( category );
             createdDataSetCount++;
             dataSet->blockSignals( false );
         }
     }
-    
-    while ( d->dataSets.size() > createdDataSetCount ) {
-    	DataSet *dataSet = d->dataSets.takeLast();
 
-    	// TODO: Restore attached axis when dataset is re-inserted?
-    	if ( dataSet->attachedAxis() )
-    	    dataSet->attachedAxis()->detachDataSet( dataSet, true );
-
-        d->removedDataSets.append( dataSet );
-    }
+    return createdDataSets;
 }
 
 
@@ -329,6 +372,8 @@ void ChartProxyModel::setSourceModel( QAbstractItemModel *sourceModel )
     beginResetModel();
 
     if ( this->sourceModel() ) {
+        disconnect( this->sourceModel(), SIGNAL( modelReset() ),
+                    this,                SLOT( slotModelReset() ) );
         disconnect( this->sourceModel(), SIGNAL( dataChanged( const QModelIndex&, const QModelIndex& ) ),
                     this,                SLOT( dataChanged( const QModelIndex&, const QModelIndex& ) ) );
         disconnect( this->sourceModel(), SIGNAL( rowsInserted( const QModelIndex&, int, int ) ),
@@ -342,6 +387,8 @@ void ChartProxyModel::setSourceModel( QAbstractItemModel *sourceModel )
     }
 
     if ( sourceModel ) {
+        connect( sourceModel, SIGNAL( modelReset() ),
+                 this,        SLOT( slotModelReset() ) );
         connect( sourceModel, SIGNAL( dataChanged( const QModelIndex&, const QModelIndex& ) ),
                  this,        SLOT( dataChanged( const QModelIndex&, const QModelIndex& ) ) );
         connect( sourceModel, SIGNAL( rowsInserted( const QModelIndex&, int, int ) ),
@@ -363,23 +410,8 @@ void ChartProxyModel::setSourceModel( QAbstractItemModel *sourceModel )
 void ChartProxyModel::setSourceModel( QAbstractItemModel *model,
                                       const QVector<QRect> &selection )
 {
-    // FIXME: Why are we not using ChartProxyModel::setSourceModel() here to connect all signals
-    // after setting d->selection ?
-    beginResetModel();
-
-    if ( sourceModel() )
-        disconnect( sourceModel(), SIGNAL( dataChanged( const QModelIndex&, const QModelIndex& ) ),
-                    this,          SLOT( dataChanged( const QModelIndex&, const QModelIndex& ) ) );
-
-    connect( model, SIGNAL( dataChanged( const QModelIndex&, const QModelIndex& ) ),
-             this,  SLOT( dataChanged( const QModelIndex&, const QModelIndex& ) ) );
-    
     d->selection = selection;
-
-    QAbstractProxyModel::setSourceModel( model );
-
-    rebuildDataMap();
-    endResetModel();
+    setSourceModel( model );
 }
 
 void ChartProxyModel::setSelection( const QVector<QRect> &selection )
@@ -396,7 +428,7 @@ void ChartProxyModel::saveOdf( KoShapeSavingContext &context ) const
     foreach ( DataSet *dataSet, d->dataSets ) {
         bodyWriter.startElement( "chart:series" );
         
-        KoGenStyle style( KoGenStyle::StyleGraphicAuto, "chart" );
+        KoGenStyle style( KoGenStyle::GraphicAutoStyle, "chart" );
         
         if ( dataSet->chartType() != LastChartType )
             style.addProperty( "chart:family", ODF_CHARTTYPES[ dataSet->chartType() ] );
@@ -411,7 +443,7 @@ void ChartProxyModel::saveOdf( KoShapeSavingContext &context ) const
         bodyWriter.addAttribute( "chart:values-cell-range-address", prefix + dataSet->yDataRegionString() );
         bodyWriter.addAttribute( "chart:label-cell-address", prefix + dataSet->labelDataRegionString() );
         
-        const QString styleName = mainStyles.lookup( style, "ch", KoGenStyles::ForceNumbering );
+        const QString styleName = mainStyles.insert( style, "ch" );
         bodyWriter.addAttribute( "chart:style-name", styleName );
         
         bodyWriter.endElement(); // chart:series
@@ -427,7 +459,22 @@ bool ChartProxyModel::loadOdf( const KoXmlElement &element,
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
     styleStack.save();
 
-    d->dataSets.clear();
+    invalidateDataSets();
+
+    QList<DataSet*> createdDataSets;
+    int loadedDataSetCount = 0;
+
+    // A cell range for all data is optional.
+    // If it is specified, use createDataSetsFromRegion() to automatically
+    // turn this data region into consecutive data series.
+    // If cell ranges are in addition specified for one or more of these
+    // data series, they'll be overwritten by these values.
+    if ( element.hasAttributeNS( KoXmlNS::table, "cell-range-address" ) )
+    {
+        QString cellRangeAddress = element.attributeNS( KoXmlNS::table, "cell-range-address" );
+        setSelection( CellRegion::stringToRegion( cellRangeAddress ) );
+        createdDataSets = createDataSetsFromRegion( d->removedDataSets );
+    }
 
     KoXmlElement n;
     forEachElement ( n, element ) {
@@ -435,69 +482,23 @@ bool ChartProxyModel::loadOdf( const KoXmlElement &element,
             continue;
 
         if ( n.localName() == "series" ) {
-            DataSet *dataSet = new DataSet( this );
-            dataSet->setNumber( d->dataSets.size() );
+            DataSet *dataSet;
+            if ( loadedDataSetCount < createdDataSets.size() ) {
+                dataSet = createdDataSets[loadedDataSetCount];
+            } else {
+                dataSet = new DataSet( this );
+                dataSet->setNumber( d->dataSets.size() );
+            }
             d->dataSets.append( dataSet );
+            dataSet->loadOdf( n, context );
 
-            if ( n.hasAttributeNS( KoXmlNS::chart, "style-name" ) ) {
-                //qDebug() << "HAS style-name:" << n.attributeNS( KoXmlNS::chart, "style-name" );
-                styleStack.clear();
-                context.odfLoadingContext().fillStyleStack( n, KoXmlNS::chart, "style-name", "chart" );
-
-                //styleStack.setTypeProperties( "chart" );
-
-                // FIXME: Load Pie explode factors
-                //if ( styleStack.hasProperty( KoXmlNS::chart, "pie-offset" ) )
-                //    setPieExplodeFactor( dataSet, styleStack.property( KoXmlNS::chart, "pie-offset" ).toInt() );
-
-                styleStack.setTypeProperties( "graphic" );
-
-                if ( styleStack.hasProperty( KoXmlNS::draw, "stroke" ) ) {
-                    qDebug() << "HAS stroke";
-                    QString stroke = styleStack.property( KoXmlNS::draw, "stroke" );
-                    if( stroke == "solid" || stroke == "dash" ) {
-                        QPen pen = KoOdfGraphicStyles::loadOdfStrokeStyle( styleStack, stroke, context.odfLoadingContext().stylesReader() );
-                        dataSet->setPen( pen );
-                    }
-                }
-
-                if ( styleStack.hasProperty( KoXmlNS::draw, "fill" ) ) {
-                    //qDebug() << "HAS fill";
-                    QString fill = styleStack.property( KoXmlNS::draw, "fill" );
-                    QBrush brush;
-                    if ( fill == "solid" || fill == "hatch" ) {
-                        brush = KoOdfGraphicStyles::loadOdfFillStyle( styleStack, fill, context.odfLoadingContext().stylesReader() );
-                    } else if ( fill == "gradient" ) {
-                        brush = KoOdfGraphicStyles::loadOdfGradientStyle( styleStack, context.odfLoadingContext().stylesReader(), QSizeF( 5.0, 60.0 ) );
-                    } else if ( fill == "bitmap" )
-                        brush = KoOdfGraphicStyles::loadOdfPatternStyle( styleStack, context.odfLoadingContext(), QSizeF( 5.0, 60.0 ) );
-                    dataSet->setBrush( brush );
-                } else {
-                    dataSet->setColor( defaultDataSetColor( dataSet->number() ) );
-                }
-            }
-
-            if ( n.hasAttributeNS( KoXmlNS::chart, "values-cell-range-address" ) ) {
-                const QString region = n.attributeNS( KoXmlNS::chart, "values-cell-range-address", QString() );
-                dataSet->setYDataRegionString( region );
-            }
-            if ( n.hasAttributeNS( KoXmlNS::chart, "label-cell-address" ) ) {
-                const QString region = n.attributeNS( KoXmlNS::chart, "label-cell-address", QString() );
-                dataSet->setLabelDataRegionString( region );
-            }
-
-            KoXmlElement m;
-            forEachElement ( m, n ) {
-                if ( m.namespaceURI() != KoXmlNS::chart )
-                    continue;
-                // FIXME: Load data points
-            }
+            loadedDataSetCount++;
         } else {
             qWarning() << "ChartProxyModel::loadOdf(): Unknown tag name \"" << n.localName() << "\"";
         }
     }
 
-    rebuildDataMap();
+    //rebuildDataMap();
     endResetModel();
 
     styleStack.restore();
@@ -535,22 +536,22 @@ void ChartProxyModel::dataChanged( const QModelIndex& topLeft, const QModelIndex
     QRect dataChangedRect = QRect( topLeftPoint,
                                    QSize( bottomRightPoint.x() - topLeftPoint.x() + 1,
                                           bottomRightPoint.y() - topLeftPoint.y() + 1 ) );
-    
+
     foreach ( DataSet *dataSet, d->dataSets ) {
-        bool intersects = false;
-        QRect changedRect;
-        foreach ( const QRect &rect, dataSet->yDataRegion().rects() ) {
-            if ( rect.intersects( dataChangedRect ) ) {
-                changedRect |= rect.intersected( dataChangedRect );
-                intersects = true;
+        if ( dataSet->xDataRegion().intersects( dataChangedRect ) )
+            dataSet->xDataChanged( dataSet->xDataRegion().intersected( dataChangedRect ).boundingRect() );
 
-                break;
-            }
-        }
+        if ( dataSet->yDataRegion().intersects( dataChangedRect ) )
+            dataSet->yDataChanged( dataSet->yDataRegion().intersected( dataChangedRect ).boundingRect() );
 
-        if ( intersects ) {
-            dataSet->yDataChanged( changedRect );
-        }
+        if ( dataSet->categoryDataRegion().intersects( dataChangedRect ) )
+            dataSet->categoryDataChanged( dataSet->categoryDataRegion().intersected( dataChangedRect ).boundingRect() );
+
+        if ( dataSet->labelDataRegion().intersects( dataChangedRect ) )
+            dataSet->labelDataChanged( dataSet->labelDataRegion().intersected( dataChangedRect ).boundingRect() );
+
+        if ( dataSet->customDataRegion().intersects( dataChangedRect ) )
+            dataSet->customDataChanged( dataSet->customDataRegion().intersected( dataChangedRect ).boundingRect() );
     }
 	
     emit dataChanged();
@@ -639,69 +640,14 @@ QModelIndex ChartProxyModel::parent( const QModelIndex &index ) const
 
 QModelIndex ChartProxyModel::mapFromSource( const QModelIndex &sourceIndex ) const
 {
-    int  row;
-    int  column;
-
-    if ( d->dataDirection == Qt::Horizontal ) {
-        row    = sourceIndex.row();
-        column = sourceIndex.column();
-
-        if ( d->firstRowIsLabel )
-            row--;
-        if ( d->firstColumnIsLabel )
-            column--;
-        
-        // Find the first occurrence of row in the map
-        for ( int i = 0; i < d->dataMap.size(); i++ ) {
-            if ( d->dataMap[i] == row ) {
-                row = i;
-                break;
-            }
-        }
-    }
-    else {
-        // d->dataDirection == Qt::Vertical here
-
-        row    = sourceIndex.column();
-        column = sourceIndex.row();
-
-        if ( d->firstRowIsLabel )
-            row--;
-        if ( d->firstColumnIsLabel )
-            column--;
-        
-        // Find the first occurrence of column in the map
-        for ( int i = 0; i < d->dataMap.size(); i++ ) {
-            if ( d->dataMap[i] == column ) {
-                column = i;
-                break;
-            }
-        }
-    }
-    
-    return sourceModel()->index( row, column );
+    Q_UNUSED( sourceIndex );
+    return QModelIndex();
 }
 
 QModelIndex ChartProxyModel::mapToSource( const QModelIndex &proxyIndex ) const
 {
-    int  row;
-    int  column;
-
-    if ( d->dataDirection == Qt::Horizontal ) {
-        row    = d->dataMap[ proxyIndex.row() ];
-        column = proxyIndex.column();
-    }
-    else {
-        row    = proxyIndex.column();
-        column = d->dataMap[ proxyIndex.row() ];
-    }
-
-    if ( d->firstRowIsLabel )
-        row++;
-    if ( d->firstColumnIsLabel )
-        column++;
-
-    return sourceModel()->index( row, column );
+    Q_UNUSED( proxyIndex );
+    return QModelIndex();
 }
 
 
@@ -839,6 +785,9 @@ void ChartProxyModel::invalidateDataSets()
             dataSet->attachedAxis()->detachDataSet( dataSet, true );
         }
     }
+
+    d->removedDataSets = d->dataSets;
+    d->dataSets.clear();
 }
 
 void ChartProxyModel::setDataDirection( Qt::Orientation orientation )
@@ -878,6 +827,16 @@ bool ChartProxyModel::firstColumnIsLabel() const
     return d->firstColumnIsLabel;
 }
 
+QString ChartProxyModel::categoryDataRegion() const
+{
+    return d->categoryDataRegion;
+}
+
+void ChartProxyModel::setCategoryDataRegion(const QString& region)
+{
+    d->categoryDataRegion = region;
+}
+
 QList<DataSet*> ChartProxyModel::dataSets() const
 {
     return d->dataSets;
@@ -931,5 +890,10 @@ void ChartProxyModel::slotColumnsRemoved( const QModelIndex &parent,
     endResetModel();
 }
 
+void ChartProxyModel::slotModelReset()
+{
+    rebuildDataMap();
+    reset(); // propagate
+}
 
 #include "ChartProxyModel.moc"
