@@ -2,6 +2,9 @@
    Copyright (C) 1998-2002 The KSpread Team <koffice-devel@kde.org>
    Copyright (C) 2005 Tomas Mecir <mecirt@gmail.com>
    Copyright 2007 Sascha Pfau <MrPeacock@gmail.com>
+   Copyright (C) 2010 Carlos Licea <carlos@kdab.com>
+   Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+     Contact: Suresh Chande suresh.chande@nokia.com
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -34,8 +37,10 @@
 #include "ValueCalc.h"
 #include "ValueConverter.h"
 
-// these two are needed for SUBTOTAL:
+// needed for SUBTOTAL:
 #include "Cell.h"
+#include "Sheet.h"
+#include "RowColumnFormat.h"
 
 // needed by MDETERM and MINVERSE
 #include <Eigen/LU>
@@ -887,17 +892,33 @@ Value func_mround(valVector args, ValueCalc *calc, FuncExtra *)
 // Function: ROUNDDOWN
 Value func_rounddown(valVector args, ValueCalc *calc, FuncExtra *)
 {
-    if (args.count() == 2)
-        return calc->roundDown(args[0], args[1]);
-    return calc->roundDown(args[0], 0);
+    if (args.count() == 2) {
+        if (calc->greater(args[0], 0.0))
+            return calc->roundDown(args[0], args[1]);
+        else
+            return calc->roundUp(args[0], args[1]);
+    }
+
+    if (calc->greater(args[0], 0.0))
+        return calc->roundDown(args[0], 0);
+    else
+        return calc->roundUp(args[0], 0);
 }
 
 // Function: ROUNDUP
 Value func_roundup(valVector args, ValueCalc *calc, FuncExtra *)
 {
-    if (args.count() == 2)
-        return calc->roundUp(args[0], args[1]);
-    return calc->roundUp(args[0], 0);
+    if (args.count() == 2) {
+        if (calc->greater(args[0], 0.0))
+            return calc->roundUp(args[0], args[1]);
+        else
+            return calc->roundDown(args[0], args[1]);
+    }
+
+    if (calc->greater(args[0], 0.0))
+        return calc->roundUp(args[0], 0);
+    else
+        return calc->roundDown(args[0], 0);
 }
 
 // Function: ROUND
@@ -1185,19 +1206,30 @@ Value func_subtotal(valVector args, ValueCalc *calc, FuncExtra *e)
         c2 = e->ranges[1].col2;
     }
 
-    // if we have a range, run through it, and put an empty value to the place
-    // of all occurrences of the SUBTOTAL function
+    // exclude manually hidden rows. http://tools.oasis-open.org/issues/browse/OFFICE-2030
+    bool excludeHiddenRows = false;
+    if(function > 100) {
+        excludeHiddenRows = true;
+        function = function % 100; // translate e.g. 106 to 6.
+    }
+
+    // run through the cells in the selected range
     Value empty;
     if ((r1 > 0) && (c1 > 0) && (r2 > 0) && (c2 > 0)) {
-        for (int r = r1; r <= r2; ++r)
+        for (int r = r1; r <= r2; ++r) {
+            const bool setAllEmpty = excludeHiddenRows && e->sheet->rowFormat(r)->isHidden();
             for (int c = c1; c <= c2; ++c) {
-                Cell cell(e->sheet, c, r);
-                if (cell.isDefault())
+                // put an empty value to all cells in a hidden row
+                if(setAllEmpty) {
+                    range.setElement(c - c1, r - r1, empty);
                     continue;
-                if (cell.isFormula() && cell.userInput().indexOf("SUBTOTAL", 0, Qt::CaseInsensitive) != -1)
-                    // cell contains the word SUBTOTAL - replace value with empty
+                }
+                Cell cell(e->sheet, c, r);
+                // put an empty value to the place of all occurrences of the SUBTOTAL function
+                if (!cell.isDefault() && cell.isFormula() && cell.userInput().indexOf("SUBTOTAL", 0, Qt::CaseInsensitive) != -1)
                     range.setElement(c - c1, r - r1, empty);
             }
+        }
     }
 
     // Good. Now we can execute the necessary function on the range.
@@ -1235,14 +1267,14 @@ Value func_subtotal(valVector args, ValueCalc *calc, FuncExtra *e)
     case 10: // Var
         f = FunctionRepository::self()->function("VAR");
         if (!f) return Value::errorVALUE();
-        a.reserve(1);
+        a.resize(1);
         a[0] = range;
         res = f->exec(a, calc, 0);
         break;
     case 11: // VarP
         f = FunctionRepository::self()->function("VARP");
         if (!f) return Value::errorVALUE();
-        a.reserve(1);
+        a.resize(1);
         a[0] = range;
         res = f->exec(a, calc, 0);
         break;
