@@ -28,6 +28,10 @@
 #include "KoMainWindow.h"
 #include "KoFilterManager.h"
 #include "KoDocumentInfo.h"
+#include "rdf/KoDocumentRdfBase.h"
+#ifdef SHOULD_BUILD_RDF
+#include "rdf/KoDocumentRdf.h"
+#endif
 #include "KoOdfStylesReader.h"
 #include "KoOdfReadStore.h"
 #include "KoOdfWriteStore.h"
@@ -35,12 +39,12 @@
 #include "KoXmlNS.h"
 #include "KoOpenPane.h"
 #include "KoApplication.h"
-#include "KoUndoStack.h"
 
 #include <KoDpi.h>
 #include <KoXmlWriter.h>
 
 #include <kdialog.h>
+#include <KUndoStack>
 #include <kfileitem.h>
 #include <kio/job.h>
 #include <kio/jobuidelegate.h>
@@ -138,6 +142,11 @@ public:
 
     KoViewWrapperWidget *wrapperWidget;
     KoDocumentInfo *docInfo;
+#ifdef SHOULD_BUILD_RDF
+    KoDocumentRdf *docRdf;
+#else
+    KoDocumentRdfBase *docRdf;
+#endif
 
     KoUnit unit;
 
@@ -172,7 +181,7 @@ public:
     QString templateType;
     QList<KoVersionInfo> versionInfo;
 
-    KoUndoStack* undoStack;
+    KUndoStack *undoStack;
 
     KoGridData gridData;
     KoGuidesData guidesData;
@@ -282,6 +291,10 @@ KoDocument::KoDocument(QWidget * parentWidget, QObject* parent, bool singleViewM
     }
 
     d->docInfo = new KoDocumentInfo(this);
+    d->docRdf = 0;
+#ifdef SHOULD_BUILD_RDF
+    d->docRdf  = new KoDocumentRdf(this);
+#endif
 
     d->pageLayout.width = 0;
     d->pageLayout.height = 0;
@@ -290,7 +303,7 @@ KoDocument::KoDocument(QWidget * parentWidget, QObject* parent, bool singleViewM
     d->pageLayout.leftMargin = 0;
     d->pageLayout.rightMargin = 0;
 
-    d->undoStack = new KoUndoStack(this);
+    d->undoStack = new KUndoStack(this);
     d->undoStack->createUndoAction(actionCollection());
     d->undoStack->createRedoAction(actionCollection());
 
@@ -704,6 +717,20 @@ KoDocumentInfo *KoDocument::documentInfo() const
     return d->docInfo;
 }
 
+KoDocumentRdf *KoDocument::documentRdf() const
+{
+#ifdef SHOULD_BUILD_RDF
+    return d->docRdf;
+#endif
+    return 0;
+}
+
+KoDocumentRdfBase *KoDocument::documentRdfBase() const
+{
+    return d->docRdf;
+}
+
+
 void KoDocument::paintEverything(QPainter &painter, const QRect &rect, KoView *view)
 {
     Q_UNUSED(view);
@@ -813,6 +840,12 @@ bool KoDocument::saveNativeFormatODF(KoStore* store, const QByteArray &mimeType)
         return false;
     }
 
+    if (d->docRdf && !d->docRdf->saveOasis(store, manifestWriter)) {
+        d->lastErrorMessage = i18n("Not able to write RDF metadata. Partition full?");
+        delete store;
+        return false;
+    }
+
     if (store->open("Thumbnails/thumbnail.png")) {
         if (!saveOasisPreview(store, manifestWriter) || !store->close()) {
             d->lastErrorMessage = i18n("Error while trying to write '%1'. Partition full?", QString("Thumbnails/thumbnail.png"));
@@ -1588,8 +1621,16 @@ bool KoDocument::loadNativeFormatFromStoreInternal(KoStore * store)
         d->docInfo = new KoDocumentInfo(this);
     }
 
+    if (oasis && store->hasFile("manifest.rdf") && d->docRdf) {
+        d->docRdf->loadOasis(store);
+    }
+
     if (oasis && store->hasFile("VersionList.xml")) {
-        KMessageBox::information(0, i18n("This document contains several versions. Go to File->Versions to open an old version."), QString(), "informVersionsAtLoading");
+        KNotification *notify = new KNotification("DocumentHasVersions");
+        notify->setText(i18n("Document <i>%1</i> contains several versions. Go to File->Versions to open an old version.", store->urlOfStore().url()));
+        notify->addContext("url", store->urlOfStore().url());
+        QTimer::singleShot(0, notify, SLOT(sendEvent()));
+
         KoXmlDocument versionInfo;
         KoOdfReadStore oasisStore(store);
         if (oasisStore.loadAndParse("VersionList.xml", versionInfo, d->lastErrorMessage)) {
@@ -2222,7 +2263,7 @@ KoUnit KoDocument::unit() const
     return d->unit;
 }
 
-void KoDocument::setUnit(KoUnit unit)
+void KoDocument::setUnit(const KoUnit &unit)
 {
     if (d->unit != unit) {
         d->unit = unit;
@@ -2380,7 +2421,7 @@ QList<KoVersionInfo> & KoDocument::versionList()
     return d->versionInfo;
 }
 
-KoUndoStack* KoDocument::undoStack()
+KUndoStack* KoDocument::undoStack()
 {
     return d->undoStack;
 }
@@ -2424,17 +2465,15 @@ KoGuidesData &KoDocument::guidesData()
 
 KoMainWindow* KoDocument::currentShell()
 {
-    QWidget* widget = qApp->activeWindow();
-    if (!widget) return 0;
-
-    while(qobject_cast<KoMainWindow*>(widget) == 0 && widget->parent() && widget->parent()->inherits("QWidget")) {
-        widget = qobject_cast<QWidget*>(widget->parent());
+    QWidget *widget = qApp->activeWindow();
+    KoMainWindow *shell = qobject_cast<KoMainWindow*>(widget);
+    while (!shell && widget) {
+        widget = widget->parentWidget();
+        shell = qobject_cast<KoMainWindow*>(widget);
     }
 
-    KoMainWindow* shell = qobject_cast<KoMainWindow*>(widget);
-    if (!shell) {
+    if (!shell)
         shell = d->shells.first();
-    }
     return shell;
 }
 
